@@ -258,6 +258,11 @@ and do_fsck ?(before=false) g =
          );
 
       | dev, "ext4" ->
+         if before then (
+           (* Replay and hence repair a dirty log (RHEL-97600) *)
+           Fun.protect ~finally:g#umount_all (fun () -> g#mount_ro dev "/");
+         );
+
          g#e2fsck ~forceno:true dev
 
       | dev, "xfs" ->
@@ -271,7 +276,18 @@ and do_fsck ?(before=false) g =
            Fun.protect ~finally:g#umount_all (fun () -> g#mount_ro dev "/");
          );
 
-         if g#xfs_repair ~nomodify:true dev <> 0 then
+         (* Must specify the -n flag because we are not attempting to
+          * fix the filesystem here.
+          *)
+         let nomodify = true
+         (* xfs_repair runs out of memory in the low memory environment
+          * of the appliance unless we limit the amount of memory it will
+          * use here.
+          *)
+         and noprefetch = true
+         and maxmem = Int64.of_int (g#get_memsize () / 2) in
+
+         if g#xfs_repair ~maxmem ~noprefetch ~nomodify dev <> 0 then
            error (f_"detected errors on the XFS filesystem on %s") dev
 
       | _, _ ->
