@@ -185,32 +185,31 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
     let machine, virtio_1_0 =
       match inspect.i_arch with
       | ("i386"|"x86_64") ->
-        (try
-           let os = Libosinfo_utils.get_os_by_short_id inspect.i_osinfo in
-           let devices = os#get_devices () in
-           debug "libosinfo devices for OS \"%s\":\n%s" inspect.i_osinfo
-             (Libosinfo_utils.string_of_osinfo_device_list devices);
-           let { Libosinfo_utils.q35; vio10 } =
-             Libosinfo_utils.os_support_of_osinfo_device_list devices in
-           (if q35 then Q35 else I440FX), vio10
-         with
-         | Not_found ->
-           (* Pivot on the year 2007.  Any Linux distro from earlier than 2007
-            * should use i440fx, anything 2007 or newer should use q35.
-            *)
-           (match inspect.i_distro, inspect.i_major_version with
-            | "fedora", _ -> Q35
-            | ("rhel"|"centos"|"circle"|"scientificlinux"|"redhat-based"|
-               "oraclelinux"|"rocky"), major ->
-              if major <= 4 then I440FX else Q35
-            | ("sles"|"suse-based"|"opensuse"), major ->
-              if major < 10 then I440FX else Q35
-            | ("debian"|"ubuntu"|"linuxmint"|"kalilinux"), major ->
-              if major < 4 then I440FX else Q35
+        (match Libosinfo_utils.get_os_by_short_id inspect.i_osinfo with
+         | Some os ->
+            let devices = os#get_devices () in
+            debug "libosinfo devices for OS \"%s\":\n%s" inspect.i_osinfo
+              (Libosinfo_utils.string_of_osinfo_device_list devices);
+            ((if Libosinfo_utils.os_devices_supports_q35 devices
+              then Q35 else I440FX),
+             Libosinfo_utils.os_devices_supports_vio10 devices)
+         | None ->
+            (* Pivot on the year 2007.  Any Linux distro from earlier than 2007
+             * should use i440fx, anything 2007 or newer should use q35.
+             *)
+            (match inspect.i_distro, inspect.i_major_version with
+             | "fedora", _ -> Q35
+             | ("rhel"|"centos"|"circle"|"scientificlinux"|"redhat-based"|
+                "oraclelinux"|"rocky"), major ->
+                if major <= 4 then I440FX else Q35
+             | ("sles"|"suse-based"|"opensuse"), major ->
+                if major < 10 then I440FX else Q35
+             | ("debian"|"ubuntu"|"linuxmint"|"kalilinux"), major ->
+                if major < 4 then I440FX else Q35
 
-            (* reasonable default for all modern Linux kernels *)
-            | _, _ -> Q35
-           ), true
+             (* reasonable default for all modern Linux kernels *)
+             | _, _ -> Q35
+            ), true
         )
 
       | _ -> Virt, true
@@ -351,15 +350,15 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
       else false in
     List.iter (
       fun { G.app2_name = name } ->
-        if String.is_prefix name "vmware-tools-libraries-" then
+        if String.starts_with "vmware-tools-libraries-" name then
           List.push_front name libraries
-        else if String.is_prefix name "vmware-tools-" then
+        else if String.starts_with "vmware-tools-" name then
           List.push_front name remove
         else if name = "VMwareTools" then
           List.push_front name remove
-        else if String.is_prefix name "kmod-vmware-tools" then
+        else if String.starts_with "kmod-vmware-tools" name then
           List.push_front name remove
-        else if String.is_prefix name "open-vm-tools-" then
+        else if String.starts_with "open-vm-tools-" name then
           List.push_front name remove
         else if name = "open-vm-tools" && not has_ubuntu_server then
           List.push_front name remove
@@ -385,7 +384,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
             let provides =
               List.filter (
                 fun s ->
-                  not (library = s || String.is_prefix s (library ^ " = "))
+                  not (library = s || String.starts_with (library ^ " = ") s)
               ) provides in
 
             (* If the package provides something other than itself, then
@@ -465,7 +464,8 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
   and unconfigure_citrix () =
     let pkgs =
       List.filter (
-        fun { G.app2_name = name } -> String.is_prefix name "xe-guest-utilities"
+        fun { G.app2_name = name } ->
+          String.starts_with "xe-guest-utilities" name
       ) inspect.i_apps in
     let pkgs = List.map (fun { G.app2_name = name } -> name) pkgs in
 
@@ -1179,10 +1179,10 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
       let replace device =
         try List.assoc device map
         with Not_found ->
-          if not (String.is_prefix device "md") &&
-             not (String.is_prefix device "fd") &&
-             not (String.is_prefix device "sr") &&
-             not (String.is_prefix device "scd") &&
+          if not (String.starts_with "md" device) &&
+             not (String.starts_with "fd" device) &&
+             not (String.starts_with "sr" device) &&
+             not (String.starts_with "scd" device) &&
              device <> "cdrom" then
             warning (f_"%s references unknown device \"%s\".  You may have to \
                         fix this entry manually after conversion.")
@@ -1254,7 +1254,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
     (* Some linux uefi setups can't boot after conversion because of
        lost uefi boot entries. The uefi boot entries are stored in uefi
        NVRAM. The NVRAM content isn't a part of vm disk content and
-       usualy can't be converted alongside the vm.
+       usually can't be converted alongside the vm.
        If a vm doesn't have uefi fallback path (/EFI/BOOT/BOOT<arch>.efi)
        the vm is unbootable after conversion.
        The following code tries to make an uefi fallback path for
