@@ -142,31 +142,21 @@ let rec main () =
         if_prefix_length; if_nameservers }
   in
 
-  let root_choice = ref AskRoot in
-  let set_root_choice = function
-    | "ask" -> root_choice := AskRoot
-    | "single" -> root_choice := SingleRoot
-    | "first" -> root_choice := FirstRoot
-    | dev when String.is_prefix dev "/dev/" -> root_choice := RootDev dev
-    | s ->
-      error (f_"unknown --root option: %s") s
-  in
+  let root_choice = ref default_root_choice in
+  let set_root_choice = Types.set_root_choice root_choice in
 
   (* Other options that we handle here. *)
   let print_source = ref false in
 
-  let input_mode = ref `Not_set in
+  let input_modes =
+    Select_input.input_modes |>
+    List.map Select_input.string_of_input_mode |>
+    String.concat "|" in
+  let input_mode = ref None in
   let set_input_mode mode =
-    if !input_mode <> `Not_set then
+    if !input_mode <> None then
       error (f_"%s option used more than once on the command line") "-i";
-    match mode with
-    | "disk" | "local" -> input_mode := `Disk
-    | "libvirt" -> input_mode := `Libvirt
-    | "libvirtxml" -> input_mode := `LibvirtXML
-    | "ova" -> input_mode := `OVA
-    | "vmx" -> input_mode := `VMX
-    | s ->
-       error (f_"unknown -i option: %s") s
+    input_mode := Some (Select_input.input_mode_of_string mode)
   in
 
   let argspec = [
@@ -174,7 +164,7 @@ let rec main () =
                                     s_"Map bridge ‘in’ to ‘out’";
     [ L"block-driver" ], Getopt.String ("driver", set_string_option_once "--block-driver" block_driver),
                                     s_"Prefer 'virtio-blk' or 'virtio-scsi'";
-    [ S 'i' ],       Getopt.String ("disk|libvirt|libvirtxml|ova|vmx", set_input_mode),
+    [ S 'i' ],       Getopt.String (input_modes, set_input_mode),
                                     s_"Set input mode (default: libvirt)";
     [ M"ic" ],       Getopt.String ("uri", set_string_option_once "-ic" input_conn),
                                     s_"Libvirt URI";
@@ -269,11 +259,9 @@ read the man page virt-v2v-in-place(1).
       pr "mac-ip-option\n";
       pr "customize-ops\n";
       pr "output-xml-option\n";
-      pr "input:disk\n";
-      pr "input:libvirt\n";
-      pr "input:libvirtxml\n";
-      pr "input:ova\n";
-      pr "input:vmx\n";
+      Select_input.input_modes |>
+        List.map Select_input.string_of_input_mode |>
+        List.iter (pr "input:%s\n");
       pr "convert:linux\n";
       pr "convert:windows\n";
       List.iter (pr "ovf:%s\n") Create_ovf.ovf_flavours;
@@ -281,35 +269,12 @@ read the man page virt-v2v-in-place(1).
    | _, _ -> ()
   );
 
-  (* Get the input module. *)
+  (* Select the input module. *)
   let (module Input_module) =
-    match input_mode with
-    | `Disk -> (module Input_disk.Disk : Input.INPUT)
-    | `LibvirtXML -> (module Input_libvirt.LibvirtXML)
-    | `OVA -> (module Input_ova.OVA)
-    | `VMX -> (module Input_vmx.VMX)
-    | `Not_set | `Libvirt ->
-       match input_conn with
-       | None -> (module Input_libvirt.Libvirt_)
-       | Some orig_uri ->
-          let { Xml.uri_server = server; uri_scheme = scheme } =
-            try Xml.parse_uri orig_uri
-            with Invalid_argument msg ->
-              error (f_"could not parse '-ic %s'.  Original error \
-                        message was: %s")
-                orig_uri msg in
-
-          match server, scheme with
-          | None, _
-          | Some "", _       (* Not a remote URI. *)
-          | Some _, None     (* No scheme? *)
-          | Some _, Some "" ->
-             (module Input_libvirt.Libvirt_)
-
-          (* vCenter over https. *)
-          | Some server, Some _ ->
-             error (f_"virt-v2v-in-place does not support remote \
-                       libvirt URIs") in
+    Select_input.select_input
+      ~allow_remote:false (* forbid remote inputs *)
+      input_mode input_conn
+      None (* no -it option *) in
 
   let input_options = {
     Input.bandwidth =
@@ -350,7 +315,7 @@ read the man page virt-v2v-in-place(1).
   (* Start the input module (runs an NBD server in the background). *)
   message (f_"Setting up the source: %s")
     (Input_module.to_string input_options args);
-  let source = Input_module.setup v2vdir input_options args in
+  let source, input_disks = Input_module.setup v2vdir input_options args in
 
   (* If --print-source then print the source metadata and exit. *)
   if print_source then (
@@ -362,7 +327,7 @@ read the man page virt-v2v-in-place(1).
 
   (* Do the conversion. *)
   with_open_out (v2vdir // "convert") (fun _ -> ());
-  let inspect, target_meta = Convert.convert v2vdir conv_options source in
+  let inspect, target_meta = Convert.convert input_disks conv_options source in
   unlink (v2vdir // "convert");
 
   (* Debug the v2vdir. *)
@@ -379,7 +344,7 @@ read the man page virt-v2v-in-place(1).
     | Output_xml_to_file filename -> Some (open_out filename) in
   Option.iter (
     fun chan ->
-      let doc = create_inspector_xml v2vdir inspect target_meta in
+      let doc = create_inspector_xml input_disks inspect target_meta in
       DOM.doc_to_chan chan doc;
       Stdlib.flush chan
   ) chan;
