@@ -16,13 +16,16 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *)
 
+type input_transport = SSH | VDDK
+(** [-it] option on the command line *)
+
 type options = {
   bandwidth : Types.bandwidth option;     (* [--bandwidth] option *)
   input_conn : string option;             (* [-ic] option *)
   input_format : string option;           (* [-if] option *)
   input_options : (string * string) list; (* [-io] options *)
   input_password : string option;         (* [-ip] option *)
-  input_transport : [`SSH|`VDDK] option;  (* [-it] option *)
+  input_transport : input_transport option;(* [-it] option *)
   read_only : bool;                       (* read-only (usually true) *)
 }
 
@@ -34,10 +37,11 @@ module type INPUT = sig
   val query_input_options : unit -> unit
   (** When the user passes [-io ?] this is used to print help. *)
 
-  val setup : string -> options -> string list -> Types.source
+  val setup : string -> options -> string list ->
+              Types.source * NBD_URI.t list
   (** [setup dir options args]
 
       Set up the input mode.  Examines the source and extracts
-      source metadata ([Types.source]).  Creates a disk pipeline
-      [dir // "inX"] for each input disk. *)
+      source metadata ([Types.source]).  Also creates a disk pipeline
+      for each input disk and returns the list of NBD URIs. *)
 end
