@@ -29,15 +29,15 @@ open Utils
 open Name_from_disk
 
 type vmx_source =
-  | File of string              (* local file or NFS *)
-  | SSH of Nbdkit_ssh.password option * Xml.uri (* SSH URI *)
+  | VMXSourceFile of string     (* local file or NFS *)
+  | VMXSourceSSH of Nbdkit_ssh.password option * Xml.uri (* SSH URI *)
 
-(* The single filename on the command line is intepreted either as
+(* The single filename on the command line is interpreted either as
  * a local file or a remote SSH URI (only if ‘-it ssh’).
  *)
 let vmx_source_of_arg input_password input_transport arg =
   match input_transport, arg with
-  | None, arg -> File arg
+  | None, arg -> VMXSourceFile arg
   | Some `SSH, arg ->
      let uri =
        try Xml.parse_uri arg
@@ -49,7 +49,7 @@ let vmx_source_of_arg input_password input_transport arg =
        error (f_"vmx URI remote server name omitted");
      if uri.Xml.uri_path = None || uri.Xml.uri_path = Some "/" then
        error (f_"vmx URI path component looks incorrect");
-     SSH (input_password, uri)
+     VMXSourceSSH (input_password, uri)
 
 let rec find_disks vmx vmx_source =
   (* Set the s_disk_id field to an incrementing number. *)
@@ -111,8 +111,9 @@ and find_nvme_disks vmx vmx_source =
  *   sata0.pciSlotNumber = "33"
  *   sata0:3.fileName = "win2019_1.vmdk"
  *
- * The "deviceType" field must be absent; that field is only used for various
- * CD-ROM types.
+ * The "deviceType" field must be absent or "disk".  Other types here
+ * include "cdrom-raw" and others indicating a CD-ROM, which we should
+ * ignore.
  *)
 and find_sata_disks vmx vmx_source =
   let get_sata_controller_target ns =
@@ -122,7 +123,7 @@ and find_sata_disks vmx vmx_source =
     try ignore (get_sata_controller_target ns); true
     with Scanf.Scan_failure _ | End_of_file | Failure _ -> false
   in
-  let sata_device_types = [ None ] in
+  let sata_device_types = [ Some "disk"; None ] in
   let sata_controller = Source_SATA in
 
   find_hdds vmx vmx_source
@@ -182,7 +183,7 @@ and find_hdds vmx vmx_source
            Some (c, t, s, filename)
         | _ -> None
     ) hdds in
-  let hdds = List.filter_map identity hdds in
+  let hdds = List.filter_map Fun.id hdds in
 
   (* We don't have a way to return the controllers and targets, so
    * just make sure the disks are sorted into order, since Parse_vmx
@@ -242,7 +243,7 @@ and find_removables vmx =
            Some s
         | _ -> None
     ) devs in
-  let devs = List.filter_map identity devs in
+  let devs = List.filter_map Fun.id devs in
 
   (* Sort by slot. *)
   let devs =
@@ -314,7 +315,7 @@ and find_nics vmx =
                    s_vnet_type = vnet_type })
         | _ -> None
     ) nics in
-  let nics = List.filter_map identity nics in
+  let nics = List.filter_map Fun.id nics in
 
   (* Sort by port. *)
   let nics = List.sort compare nics in
@@ -333,8 +334,8 @@ let parse_domain_from_vmx vmx_source =
    *)
   let vmx =
     match vmx_source with
-    | File filename -> Parse_vmx.parse_file filename
-    | SSH (password, uri) ->
+    | VMXSourceFile filename -> Parse_vmx.parse_file filename
+    | VMXSourceSSH (password, uri) ->
        let server =
          match uri.uri_server with
          | None -> assert false (* checked by vmx_source_of_arg *)
@@ -358,8 +359,8 @@ let parse_domain_from_vmx vmx_source =
     | None ->
        warning (f_"no displayName key found in VMX file");
        match vmx_source with
-       | File filename -> name_from_disk filename
-       | SSH (_, uri) ->
+       | VMXSourceFile filename -> name_from_disk filename
+       | VMXSourceSSH (_, uri) ->
           match uri.uri_path with
           | None -> assert false (* checked by vmx_source_of_arg *)
           | Some path -> name_from_disk path in
