@@ -16,7 +16,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *)
 
-val create_inspector_xml : string -> Types.inspect -> Types.target_meta ->
+val create_inspector_xml : NBD_URI.t list -> Types.inspect ->
+                           Types.target_meta ->
                            DOM.doc
 (** Create the XML output of virt-v2v-inspector which contains the
     post-conversion metadata. *)
