@@ -34,11 +34,8 @@ val datadir : string
 val host_cpu : string
 (** The configure value [@host_cpu@] *)
 
-val nbdkit_python_plugin : string
-(** Return the name of the nbdkit python plugin used by
-    [virt-v2v -o rhv-upload].
+val nbdcopy : string
+(** The location of the nbdcopy program, from configure value [@NBDCOPY@] *)
 
-    As above this must also be the Python 3 version of the plugin,
-    unless you change it.  The configure command to change this is:
-
-    [./configure --with-virt-v2v-nbdkit-python-plugin=...] *)
+val nbdinfo : string
+(** The location of the nbdinfo program, from configure value [@NBDINFO@] *)
