@@ -65,6 +65,11 @@ let probe_plugin name =
   let cmd = sprintf "nbdkit %s --version >/dev/null 2>&1" (quote name) in
   Sys.command cmd = 0
 
+let probe_plugin_parameter name regex =
+  let cmd = sprintf "nbdkit %s --help | grep -sq %s"
+              (quote name) (quote regex) in
+  Sys.command cmd = 0
+
 let probe_filter name =
   let cmd = sprintf "nbdkit null --filter=%s --version >/dev/null 2>&1"
               (quote name) in
@@ -139,11 +144,8 @@ let run_unix socket cmd =
     add_arg "system_u:object_r:svirt_socket_t:s0"
   );
 
-  (* Reduce verbosity in nbdkit >= 1.17.4. *)
-  let version = version () in
-  if version >= (1, 17, 4) then (
-    add_arg "-D"; add_arg "nbdkit.backend.datapath=0"
-  );
+  (* Reduce verbosity in nbdkit. *)
+  add_arg "-D"; add_arg "nbdkit.backend.datapath=0";
 
   List.iter (
     fun (name, value) ->
