@@ -54,6 +54,21 @@ let uri_quote str =
   done;
   String.concat "" (List.rev !xs)
 
+(* Escape characters like [?] and [*] which are special for fnmatch(3). *)
+let fnmatch_escape str =
+  let len = String.length str in
+  let xs = ref [] in
+  for i = 0 to len-1 do
+    xs :=
+      (match str.[i] with
+       | '[' | ']' | '?' | '*' as c ->
+          sprintf "\\%c" c
+       | c ->
+          String.make 1 c
+      ) :: !xs
+  done;
+  String.concat "" (List.rev !xs)
+
 (* Map guest architecture found by inspection to the architecture
  * that KVM must emulate.  Note for x86 we assume a 64 bit hypervisor.
  *)
@@ -135,7 +150,7 @@ let rec chown_for_libvirt_rhbz_1045069 file =
   if running_as_root && backend_is_libvirt () then (
     let user = Option.value ~default:"qemu" (libvirt_qemu_user ()) in
     let uid =
-      if String.is_prefix user "+" then
+      if String.starts_with "+" user then
         int_of_string (String.sub user 1 (String.length user - 1))
       else
         (Unix.getpwnam user).pw_uid in
@@ -180,6 +195,14 @@ let error_if_no_ssh_agent () =
               is not set).  This is required by qemu to do passwordless \
               ssh access.  See the virt-v2v(1) man page for more information.")
 
+let nbdcopy_supports_blkhash =
+  let check =
+    lazy (
+      let cmd = sprintf "%s --help | grep -sq -- --blkhash" Config.nbdcopy in
+      0 = Sys.command cmd
+    ) in
+  fun () -> Lazy.force check
+
 (* Create the directory containing inX and outX sockets. *)
 let create_v2v_directory () =
   let d = Mkdtemp.temp_dir "v2v." in
@@ -196,23 +219,22 @@ let rec wait_for_file filename timeout =
     wait_for_file filename (timeout-1)
   )
 
-let with_nbd_connect_unix ?(meta_contexts = []) ~socket f =
+let with_nbd_connect_uri ?(meta_contexts = []) ~uri f =
   let nbd = NBD.create () in
-  protect
-    ~f:(fun () ->
+  Fun.protect
+    (fun () ->
           NBD.set_debug nbd (verbose ());
           List.iter (NBD.add_meta_context nbd) meta_contexts;
-          NBD.connect_unix nbd socket;
-          protect
-            ~f:(fun () -> f nbd)
+          NBD.connect_uri nbd uri;
+          Fun.protect
+            (fun () -> f nbd)
             ~finally:(fun () -> NBD.shutdown nbd)
        )
     ~finally:(fun () -> NBD.close nbd)
 
-let get_disk_allocated ~dir ~disknr =
-  let socket = sprintf "%s/out%d" dir disknr
-  and alloc_ctx = "base:allocation" in
-  with_nbd_connect_unix ~socket ~meta_contexts:[alloc_ctx]
+let get_disk_allocated uri =
+  let alloc_ctx = "base:allocation" in
+  with_nbd_connect_uri ~uri ~meta_contexts:[alloc_ctx]
     (fun nbd ->
          if NBD.can_meta_context nbd alloc_ctx then (
            (* Get the list of extents, using a 2GiB chunk size as hint. *)
