@@ -56,27 +56,21 @@ let rec main () =
 
   let input_options = ref [] in
   let io_query = ref false in
-  let set_input_option_compat k v =
-    List.push_back input_options (k, v)
-  in
   let set_input_option option =
     if option = "?" then io_query := true
     else (
       let k, v = String.split "=" option in
-      set_input_option_compat k v
+      List.push_back input_options (k, v)
     )
   in
 
   let output_options = ref [] in
   let oo_query = ref false in
-  let set_output_option_compat k v =
-    List.push_back output_options (k, v)
-  in
   let set_output_option option =
     if option = "?" then oo_query := true
     else (
       let k, v = String.split "=" option in
-      set_output_option_compat k v
+      List.push_back output_options (k, v)
     )
   in
 
@@ -149,15 +143,8 @@ let rec main () =
         if_prefix_length; if_nameservers }
   in
 
-  let root_choice = ref AskRoot in
-  let set_root_choice = function
-    | "ask" -> root_choice := AskRoot
-    | "single" -> root_choice := SingleRoot
-    | "first" -> root_choice := FirstRoot
-    | dev when String.is_prefix dev "/dev/" -> root_choice := RootDev dev
-    | s ->
-      error (f_"unknown --root option: %s") s
-  in
+  let root_choice = ref default_root_choice in
+  let set_root_choice = Types.set_root_choice root_choice in
 
   let output_alloc = ref `Not_set in
   let set_output_alloc mode =
@@ -179,46 +166,26 @@ let rec main () =
   (* Other options that we handle here. *)
   let print_source = ref false in
 
-  let input_mode = ref `Not_set in
+  let input_modes =
+    Select_input.input_modes |>
+    List.map Select_input.string_of_input_mode |>
+    String.concat "|" in
+  let input_mode = ref None in
   let set_input_mode mode =
-    if !input_mode <> `Not_set then
+    if !input_mode <> None then
       error (f_"%s option used more than once on the command line") "-i";
-    match mode with
-    | "disk" | "local" -> input_mode := `Disk
-    | "libvirt" -> input_mode := `Libvirt
-    | "libvirtxml" -> input_mode := `LibvirtXML
-    | "ova" -> input_mode := `OVA
-    | "vmx" -> input_mode := `VMX
-    | s ->
-       error (f_"unknown -i option: %s") s
+    input_mode := Some (Select_input.input_mode_of_string mode)
   in
 
-  let output_mode = ref `Not_set in
+  let output_modes =
+    Select_output.output_modes |>
+    List.map Select_output.string_of_output_mode |>
+    String.concat "|" in
+  let output_mode = ref None in
   let set_output_mode mode =
-    if !output_mode <> `Not_set then
+    if !output_mode <> None then
       error (f_"%s option used more than once on the command line") "-o";
-    match mode with
-    | "glance" -> output_mode := `Glance
-    | "kubevirt" -> output_mode := `Kubevirt
-    | "libvirt" -> output_mode := `Libvirt
-    | "disk" | "local" -> output_mode := `Disk
-    | "null" -> output_mode := `Null
-    | "openstack" | "osp" | "rhosp" -> output_mode := `Openstack
-    | "ovirt" | "rhv" | "rhev" -> output_mode := `RHV
-    | "ovirt-upload" | "ovirt_upload" | "rhv-upload" | "rhv_upload" ->
-       output_mode := `RHV_Upload
-    | "qemu" -> output_mode := `QEmu
-    | "vdsm" -> output_mode := `VDSM
-    | s ->
-       error (f_"unknown -o option: %s") s
-  in
-
-  (* Options that are ignored for backwards compatibility. *)
-  let no_trim_warning _ =
-    warning (f_"the --no-trim option has been removed and now does nothing")
-  in
-  let vmtype_warning _ =
-    warning (f_"the --vmtype option has been removed and now does nothing")
+    output_mode := Some (Select_output.output_mode_of_string mode)
   in
 
   (* Options that are errors. *)
@@ -236,9 +203,7 @@ let rec main () =
       s_"Map bridge ‘in’ to ‘out’";
     [ L"block-driver" ], Getopt.String ("driver", set_string_option_once "--block-driver" block_driver),
                                     s_"Prefer 'virtio-blk' or 'virtio-scsi'";
-    [ L"compressed" ], Getopt.Unit (fun () -> set_output_option_compat "compressed" ""),
-      s_"Compress output file (-of qcow2 only)";
-    [ S 'i' ],       Getopt.String ("disk|libvirt|libvirtxml|ova|vmx", set_input_mode),
+    [ S 'i' ],       Getopt.String (input_modes, set_input_mode),
       s_"Set input mode (default: libvirt)";
     [ M"ic" ],       Getopt.String ("uri", set_string_option_once "-ic" input_conn),
                                     s_"Libvirt URI";
@@ -256,9 +221,7 @@ let rec main () =
       s_"Map NIC to network or bridge or assign static IP";
     [ S 'n'; L"network" ], Getopt.String ("in:out", add_network),
       s_"Map network ‘in’ to ‘out’";
-    [ L"no-trim" ],  Getopt.String ("-", no_trim_warning),
-      s_"Ignored for backwards compatibility";
-    [ S 'o' ],       Getopt.String ("glance|kubevirt|libvirt|local|null|openstack|qemu|rhv|rhv-upload|vdsm", set_output_mode),
+    [ S 'o' ],       Getopt.String (output_modes, set_output_mode),
       s_"Set output mode (default: libvirt)";
     [ M"oa" ],       Getopt.String ("sparse|preallocated", set_output_alloc),
                                     s_"Set output allocation mode";
@@ -274,46 +237,12 @@ let rec main () =
                                     s_"Use password from file to connect to output hypervisor";
     [ M"os" ],       Getopt.String ("storage", set_string_option_once "-os" output_storage),
                                     s_"Set output storage location";
-    [ L"password-file" ], Getopt.String ("filename", set_string_option_once "-ip" input_password),
-      s_"Same as ‘-ip filename’";
     [ L"parallel" ], Getopt.Set_int ("N", parallel),
       s_"Run up to N instances of nbdcopy in parallel";
     [ L"print-source" ], Getopt.Set print_source,
       s_"Print source and stop";
-    [ L"qemu-boot" ], Getopt.Unit (fun () -> set_output_option_compat "qemu-boot" ""),
-      s_"Boot in qemu (-o qemu only)";
     [ L"root" ],     Getopt.String ("ask|... ", set_root_choice),
       s_"How to choose root filesystem";
-    [ L"vddk-config" ], Getopt.String ("filename", set_input_option_compat "vddk-config"),
-      s_"Same as ‘-io vddk-config=filename’";
-    [ L"vddk-cookie" ], Getopt.String ("cookie", set_input_option_compat "vddk-cookie"),
-      s_"Same as ‘-io vddk-cookie=filename’";
-    [ L"vddk-libdir" ], Getopt.String ("libdir", set_input_option_compat "vddk-libdir"),
-      s_"Same as ‘-io vddk-libdir=libdir’";
-    [ L"vddk-nfchostport" ], Getopt.String ("nfchostport", set_input_option_compat "vddk-nfchostport"),
-      s_"Same as ‘-io vddk-nfchostport=nfchostport’";
-    [ L"vddk-port" ], Getopt.String ("port", set_input_option_compat "vddk-port"),
-      s_"Same as ‘-io vddk-port=port’";
-    [ L"vddk-snapshot" ], Getopt.String ("snapshot-moref", set_input_option_compat "vddk-snapshot"),
-      s_"Same as ‘-io vddk-snapshot=snapshot-moref’";
-    [ L"vddk-thumbprint" ], Getopt.String ("thumbprint", set_input_option_compat "vddk-thumbprint"),
-      s_"Same as ‘-io vddk-thumbprint=thumbprint’";
-    [ L"vddk-transports" ], Getopt.String ("transports", set_input_option_compat "vddk-transports"),
-      s_"Same as ‘-io vddk-transports=transports’";
-    [ L"vdsm-compat" ], Getopt.String ("0.10|1.1", set_output_option_compat "vdsm-compat"),
-      s_"Same as ‘-oo vdsm-compat=0.10|1.1’";
-    [ L"vdsm-image-uuid" ], Getopt.String ("uuid", set_output_option_compat "vdsm-image-uuid"),
-      s_"Same as ‘-oo vdsm-image-uuid=uuid’";
-    [ L"vdsm-vol-uuid" ], Getopt.String ("uuid", set_output_option_compat "vdsm-vol-uuid"),
-      s_"Same as ‘-oo vdsm-vol-uuid=uuid’";
-    [ L"vdsm-vm-uuid" ], Getopt.String ("uuid", set_output_option_compat "vdsm-vm-uuid"),
-      s_"Same as ‘-oo vdsm-vm-uuid=uuid’";
-    [ L"vdsm-ovf-output" ], Getopt.String ("dir", set_output_option_compat "vdsm-ovf-output"),
-      s_"Same as ‘-oo vdsm-ovf-output=dir’";
-    [ L"vdsm-ovf-flavour" ], Getopt.String ("ovirt|rhvexp", set_output_option_compat "vdsm-ovf-flavour"),
-      s_"Same as ‘-oo vdsm-ovf-flavour=flavour’";
-    [ L"vmtype" ],   Getopt.String ("-", vmtype_warning),
-      s_"Ignored for backwards compatibility";
   ] in
 
   (* Append virt-customize options. *)
@@ -332,7 +261,7 @@ let rec main () =
 virt-v2v -ic vpx://vcenter.example.com/Datacenter/esxi -os imported esx_guest
 
 virt-v2v -ic vpx://vcenter.example.com/Datacenter/esxi esx_guest \
-         -o rhv -os rhv.nfs:/export_domain --network ovirtmgmt
+         -o ovirt -os ovirt.nfs:/export_domain --network ovirtmgmt
 
 virt-v2v -i libvirtxml guest-domain.xml -o local -os /var/tmp
 
@@ -380,8 +309,8 @@ read the man page virt-v2v(1).
   let input_transport =
     match !input_transport with
     | None -> None
-    | Some "ssh" -> Some `SSH
-    | Some "vddk" -> Some `VDDK
+    | Some "ssh" -> Some Input.SSH
+    | Some "vddk" -> Some Input.VDDK
     | Some transport ->
        error (f_"unknown input transport ‘-it %s’") transport in
   let output_alloc =
@@ -416,21 +345,12 @@ read the man page virt-v2v(1).
       pr "mac-ip-option\n";
       pr "parallel-option\n";
       pr "customize-ops\n";
-      pr "input:disk\n";
-      pr "input:libvirt\n";
-      pr "input:libvirtxml\n";
-      pr "input:ova\n";
-      pr "input:vmx\n";
-      pr "output:glance\n";
-      pr "output:kubevirt\n";
-      pr "output:libvirt\n";
-      pr "output:local\n";
-      pr "output:null\n";
-      pr "output:openstack\n";
-      pr "output:qemu\n";
-      pr "output:rhv\n";
-      pr "output:rhv-upload\n";
-      pr "output:vdsm\n";
+      Select_input.input_modes |>
+        List.map Select_input.string_of_input_mode |>
+        List.iter (pr "input:%s\n");
+      Select_output.output_modes |>
+        List.map Select_output.string_of_output_mode |>
+        List.iter (pr "output:%s\n");
       pr "convert:linux\n";
       pr "convert:windows\n";
       List.iter (pr "ovf:%s\n") Create_ovf.ovf_flavours;
@@ -438,55 +358,9 @@ read the man page virt-v2v(1).
    | _, _ -> ()
   );
 
-  (* Get the input module. *)
+  (* Select the input module. *)
   let (module Input_module) =
-    match input_mode with
-    | `Disk -> (module Input_disk.Disk : Input.INPUT)
-    | `LibvirtXML -> (module Input_libvirt.LibvirtXML)
-    | `OVA -> (module Input_ova.OVA)
-    | `VMX -> (module Input_vmx.VMX)
-    | `Not_set | `Libvirt ->
-       match input_conn with
-       | None -> (module Input_libvirt.Libvirt_)
-       | Some orig_uri ->
-          let { Xml.uri_server = server; uri_scheme = scheme } =
-            try Xml.parse_uri orig_uri
-            with Invalid_argument msg ->
-              error (f_"could not parse '-ic %s'.  \
-                        Original error message was: %s")
-                orig_uri msg in
-
-          match server, scheme, input_transport with
-          | None, _, _
-            | Some "", _, _       (* Not a remote URI. *)
-
-            | Some _, None, _     (* No scheme? *)
-            | Some _, Some "", _ ->
-             (module Input_libvirt.Libvirt_)
-
-          (* vCenter over https. *)
-          | Some server, Some ("esx"|"gsx"|"vpx"), None ->
-             (module Input_vcenter_https.VCenterHTTPS)
-
-          (* vCenter or ESXi using nbdkit vddk plugin *)
-          | Some server, Some ("esx"|"gsx"|"vpx"), Some `VDDK ->
-             (module Input_vddk.VDDK)
-
-          (* Xen over SSH *)
-          | Some server, Some "xen+ssh", _ ->
-             (module Input_xen_ssh.XenSSH)
-
-          (* Old virt-v2v also supported qemu+ssh://.  However I am
-           * deliberately not supporting this in new virt-v2v.  Don't
-           * use virt-v2v if a guest already runs on KVM.
-           *)
-
-          (* Unknown remote scheme. *)
-          | Some _, Some _, _ ->
-             warning (f_"no support for remote libvirt connections to \
-                         '-ic %s'.  The conversion may fail when it tries \
-                         to read the source disks.") orig_uri;
-             (module Input_libvirt.Libvirt_) in
+    Select_input.select_input input_mode input_conn input_transport in
 
   let input_options = {
     Input.bandwidth =
@@ -511,19 +385,8 @@ read the man page virt-v2v(1).
     exit 0
   );
 
-  (* Get the output helper name and command line. *)
-  let (module Output_module) =
-    match output_mode with
-    | `Not_set | `Libvirt -> (module Output_libvirt.Libvirt_ : Output.OUTPUT)
-    | `Disk -> (module Output_disk.Disk)
-    | `Null -> (module Output_null.Null)
-    | `QEmu -> (module Output_qemu.QEMU)
-    | `Glance -> (module Output_glance.Glance)
-    | `Kubevirt -> (module Output_kubevirt.Kubevirt)
-    | `Openstack -> (module Output_openstack.Openstack)
-    | `RHV_Upload -> (module Output_rhv_upload.RHVUpload)
-    | `RHV -> (module Output_rhv.RHV)
-    | `VDSM -> (module Output_vdsm.VDSM) in
+  (* Select the output module. *)
+  let (module Output_module) = Select_output.select_output output_mode in
 
   let output_options = {
     Output.output_alloc = output_alloc;
@@ -541,14 +404,14 @@ read the man page virt-v2v(1).
     exit 0
   );
 
-  (* XXX This is a hack for -o rhv and -o vdsm where we must remove
+  (* XXX This is a hack for -o ovirt and -o vdsm where we must remove
    * the serial console for Linux conversions.  We don't do this for
-   * -o rhv-upload, even though we probably should, which would
-   * indicate that actually RHV is fine if we don't do this.
+   * -o ovirt-upload, even though we probably should, which would
+   * indicate that actually oVirt is fine if we don't do this.
    *)
   let remove_serial_console =
     match output_mode with
-    | `RHV | `VDSM -> true
+    | Some Select_output.OVirt | Some VDSM -> true
     | _ -> false in
 
   (* Get the conversion options. *)
@@ -570,7 +433,7 @@ read the man page virt-v2v(1).
   (* Start the input module (runs an NBD server in the background). *)
   message (f_"Setting up the source: %s")
     (Input_module.to_string input_options args);
-  let source = Input_module.setup v2vdir input_options args in
+  let source, input_disks = Input_module.setup v2vdir input_options args in
 
   (* If --print-source then print the source metadata and exit. *)
   if print_source then (
@@ -588,13 +451,14 @@ read the man page virt-v2v(1).
 
   (* Do the conversion. *)
   with_open_out (v2vdir // "convert") (fun _ -> ());
-  let inspect, target_meta = Convert.convert v2vdir conv_options source in
+  let inspect, target_meta = Convert.convert input_disks conv_options source in
   unlink (v2vdir // "convert");
 
   (* Start the output module (runs an NBD server in the background). *)
   message (f_"Setting up the destination: %s")
     (Output_module.to_string output_options);
-  let output_t = Output_module.setup v2vdir output_poptions source in
+  let output_t, output_disks =
+    Output_module.setup v2vdir output_poptions source input_disks in
 
   (* Debug the v2vdir. *)
   if verbose () then (
@@ -605,29 +469,27 @@ read the man page virt-v2v(1).
   (* Do the copy. *)
   with_open_out (v2vdir // "copy") (fun _ -> ());
 
-  (* Get the list of disks and corresponding sockets. *)
-  let rec loop acc i =
-    let input_socket = sprintf "%s/in%d" v2vdir i
-    and output_socket = sprintf "%s/out%d" v2vdir i in
-    if Sys.file_exists input_socket && Sys.file_exists output_socket then
-      loop ((i, input_socket, output_socket) :: acc) (i+1)
-    else
-      List.rev acc
-  in
-  let disks = ref (loop [] 0) in
-  let nr_disks = List.length !disks in
+  (* Get the list of disks and corresponding NBD URIs. *)
+  let disks =
+    List.combine input_disks output_disks |>
+    List.mapi (
+      fun i (input_uri, output_uri) ->
+        let input_uri = NBD_URI.to_uri input_uri
+        and output_uri = NBD_URI.to_uri output_uri in
+        (i, input_uri, output_uri)
+    ) in
+  let nr_disks = List.length disks in
 
   (* Copy the disks. *)
+  let disks = ref disks in
   let nbdcopy_pids = ref [] in
   let rec copy_loop () =
     if List.length !nbdcopy_pids < parallel && !disks <> [] then (
       (* Schedule another nbdcopy process. *)
-      let i, input_socket, output_socket = List.pop_front disks in
+      let i, input_uri, output_uri = List.pop_front disks in
       message (f_"Copying disk %d/%d") (i+1) nr_disks;
 
-      let request_size = Output_module.request_size
-      and input_uri = nbd_uri_of_socket input_socket
-      and output_uri = nbd_uri_of_socket output_socket in
+      let request_size = Output_module.request_size in
 
       (* In verbose mode print some information about each
        * side of the pipeline.
@@ -675,7 +537,7 @@ read the man page virt-v2v(1).
 
   (* Do the finalization step. *)
   message (f_"Creating output metadata");
-  Output_module.finalize v2vdir output_poptions output_t
+  Output_module.finalize v2vdir output_poptions output_t output_disks
     source inspect target_meta;
 
   message (f_"Finishing off");
@@ -728,13 +590,19 @@ and nbdcopy ?request_size output_alloc input_uri output_uri =
     min 64 (target_buffer_size / request_size) in
   List.push_back cmd (sprintf "--requests=%d" requests);
 
+  if verbose () && nbdcopy_supports_blkhash () then
+    List.push_back cmd "--blkhash";
+
   List.push_back cmd "--flush";
   (*List.push_back cmd "--verbose";*)
 
   if not (quiet ()) then List.push_back cmd "--progress";
   if output_alloc = Types.Preallocated then List.push_back cmd "--allocated";
 
-  let args = Array.of_list !cmd in
+  let args = !cmd in
+  debug "%s" (stringify_args args);
+
+  let args = Array.of_list args in
   match fork () with
   | 0 ->
      (* Child process (nbdcopy). *)
@@ -752,7 +620,4 @@ and nbdinfo ?(content = false) uri =
       (quote uri) in
   external_command cmd
 
-(* Convert a Unix domain socket path to an NBD URI. *)
-and nbd_uri_of_socket = sprintf "nbd+unix:///?socket=%s"
-
 let () = run_main_and_handle_errors main
